﻿using Newtonsoft.Json;
using RestSharp;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using UnivateProperties_API.Containers.Timeshare;
using UnivateProperties_API.Context;
using UnivateProperties_API.Helpers;

namespace UnivateProperties_API.Repository.Timeshare
{
    public interface IResortRepository
    {
        List<ResortDisplay> GetResortsByRegion(string regionCode);
        string GetResortDescription(string resortCode);
        List<string> GetResortImages(string resortCode);
    }

    public class ResortRepository : IResortRepository
    {
        private readonly DataContext _dbContext;

        public ResortRepository(DataContext dbContext)
        {
            _dbContext = dbContext;
        }

        public string GetResortDescription(string resortCode)
        {
            var requestDataURL = MyCommon.Reservations.Replace("ReservationsWebService.asmx", "");
            var client = new RestClient(MyCommon.Reservations)
            {
                Timeout = -1
            };
            var request = new RestRequest(Method.POST);
            request.AddHeader("charset", "utf-8");
            request.AddHeader("Content-Type", "text/xml");
            request.AddHeader("Cookie", "ASP.NET_SessionId=bis4e2xtmqjdvh2wlqfolzbt");
            request.AddParameter("text/xml"
                , string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n  <soap:Body>\r\n    <GetResortDescription xmlns=\"{3}\">\r\n      <UserCode>{0}</UserCode>\r\n      <Password>{1}</Password>\r\n      <ResortCode>{2}</ResortCode>\r\n    </GetResortDescription>\r\n  </soap:Body>\r\n</soap:Envelope>"
                , MyCommon.ReservationsUserCode
                , MyCommon.ReservationsPassword
                , resortCode
                , requestDataURL)
                , ParameterType.RequestBody);
            IRestResponse response = client.Execute(request);           

            string description = "";
            if (!string.IsNullOrEmpty(response.Content) && !response.Content.Contains("ERROR:"))
            {
                DataSet data = CreateDataSet(response.Content);
                if (data.Tables.Count > 1)
                {
                    foreach (DataRow row in data.Tables["GetResortDescriptionResponse"].Rows)
                        description = row[0].ToString();
                }
            }
            return description;
        }

        public List<string> GetResortImages(string resortCode)
        {
            var requestDataURL = MyCommon.Reservations.Replace("ReservationsWebService.asmx", "");
            var client = new RestClient(MyCommon.Reservations)
            {
                Timeout = -1
            };
            var request = new RestRequest(Method.POST);
            request.AddHeader("charset", "utf-8");
            request.AddHeader("Content-Type", "text/xml");
            request.AddHeader("Cookie", "ASP.NET_SessionId=bis4e2xtmqjdvh2wlqfolzbt");
            request.AddParameter("text/xml"
                , string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n  <soap:Body>\r\n    <GetResortImages  xmlns=\"{3}\">\r\n      <UserCode>{0}</UserCode>\r\n      <Password>{1}</Password>\r\n      <ResortCode>{2}</ResortCode>\r\n    </GetResortImages >\r\n  </soap:Body>\r\n</soap:Envelope>"
                , MyCommon.ReservationsUserCode
                , MyCommon.ReservationsPassword
                , resortCode
                , requestDataURL)
                , ParameterType.RequestBody);
            IRestResponse response = client.Execute(request);            

            var images = new List<string>();
            if (!string.IsNullOrEmpty(response.Content) && !response.Content.Contains("ERROR:"))
            {
                DataSet data = CreateDataSet(response.Content);
                if (data.Tables.Count > 1)
                {
                    foreach (DataRow row in data.Tables["string"].Rows)
                        images.Add(row[0].ToString());
                }
            }
            return images;
        }

        public DataSet CreateDataSet(string data)
        {
            try
            {
                StringReader theReader = new StringReader(data);
                DataSet theDataSet = new DataSet();
                theDataSet.ReadXml(theReader);

                return theDataSet;
            }
            catch
            {
                return null;
            }
        }

        public List<ResortDisplay> GetResortsByRegion(string regionCode)
        {
            var resorts = new List<ResortDisplay>();
            List<UniPointResorts> UniResorts = new List<UniPointResorts>();
            WeekRepository weekRepo = new WeekRepository(_dbContext);
            var weeks = weekRepo.GetDtoListAll();

            var client = new RestClient(string.Format("https://www.tradeunipoint.com/unibackend/seam/resource/rest/products/resorts/{0}", regionCode))
            {
                Timeout = -1
            };
            var request = new RestRequest(Method.GET);
            IRestResponse response = client.Execute(request);
            UniResorts = JsonConvert.DeserializeObject<List<UniPointResorts>>(response.Content);

            foreach(var resort in UniResorts)
            {
                if (!string.IsNullOrEmpty(resort.ResortCode))
                {
                    ResortDisplay desp = new ResortDisplay()
                    {
                        ResortCode = resort.ResortCode,
                        ResortName = resort.ResortName
                    };
                    var imgClient = new RestClient(string.Format("https://www.tradeunipoint.com/unibackend/seam/resource/rest/products/{0}/image/1", resort.ResortCode))
                    {
                        Timeout = -1
                    };
                    var imgRequest = new RestRequest(Method.GET);
                    IRestResponse resp = imgClient.Execute(imgRequest);
                    desp.ResortImage = resp.Content;

                    if (desp.ResortImage != "")
                        desp.ResortImage = string.Format("data:image/jpeg;base64,{0}", desp.ResortImage);

                    desp.WeeksAvailable = weeks.Where(w => w.Resort.ResortCode == resort.ResortCode).Count();

                    resorts.Add(desp);
                }
            }

            return resorts.OrderBy(r => r.ResortCode).ToList();
        }
    }
}
